-- CCFocus 1.8, by Travis Casey (efindel@gmail.com)
-- for WoW client version 3.2.x
-- 
-- Creates macros to do CC spells on your focus, and do some manipulation
-- of your focus.  See Readme.txt for more details on use.

CCFocus_ADDONNAME = "CCFocus";

CCFocus_VERSION = GetAddOnMetadata("CCFocus", "Version");

CCFocus_Options = { };

local NotInCombat = true;

local CCFocus_MACRONAME1 = "CCFocus1";
local CCFocus_MACRONAME2 = "CCFocus2";
local CCFocus_MACRONAME3 = "CCFocus3";
local CCFocus_MACRONAME4 = "CCFocus4";
local CCFocus_RAIDMACRONAME = "CCFocusRaid";

local CCWarnLoaded = nil;

local PlayerClass;

function CCFocus_Print(msg)
  if ( ( msg ) and (strlen(msg) > 0 ) ) then
    if ( DEFAULT_CHAT_FRAME ) then
      DEFAULT_CHAT_FRAME:AddMessage(msg, 1, 1, 0);
    end
  end
end

function CCFocus_PrintUsage(which)
  CCFocus_Print("  /ccf "..CCFocus_Strings["Cmd_"..which]..CCFocus_Strings["Usage_"..which]);
end

function CCFocus_OnLoad()
  this:RegisterEvent("PLAYER_ENTERING_WORLD");
  this:RegisterEvent("VARIABLES_LOADED");
  this:RegisterEvent("PLAYER_REGEN_DISABLED");
  this:RegisterEvent("PLAYER_REGEN_ENABLED");
  SlashCmdList["CCFOCUSCOMMAND"] = CCFocus_SlashHandler;
  SLASH_CCFOCUSCOMMAND1 = "/ccfocus";
  SLASH_CCFOCUSCOMMAND2 = "/ccf";
  SlashCmdList["CCFOCUSWARN"] = CCFocus_Warn;
  SLASH_CCFOCUSWARN1 = "/ccfw";
end

function CCFocus_OnEvent()
  if ( event == "PLAYER_ENTERING_WORLD" ) then
    -- check if CCWarn is loaded
    if ( IsAddOnLoaded("CCWarn") ) then
      CCWarnLoaded = true;
    end
    CCFocus_MakeMacros();
    -- we only need to do that on initial enter, so unregister
    this:UnregisterEvent("PLAYER_ENTERING_WORLD");
  elseif ( event == "VARIABLES_LOADED" ) then
    CCFocus_Print("CCFocus "..CCFocus_VERSION..CCFocus_Strings.Onload);
    -- set up default values if no values exist
    if ( not CCFocus_Options.ClearKey ) then CCFocus_Options.ClearKey = "ctrl" end;
    if ( not CCFocus_Options.SetKey ) then CCFocus_Options.SetKey = "alt" end;
    if ( not CCFocus_Options.Version ) then CCFocus_Options.Version = CCF_VERSION end;
    if ( CCFocus_Options.Version ~= CCF_VERSION ) then CCFocus_Options.Version = CCF_VERSION end;
    if ( CCFocus_Options.CCWarn == nil ) then CCFocus_Options.CCWarn = false end;
    if ( not CCFocus_RaidIcon ) then CCFocus_RaidIcon = 4 end;
    this:UnregisterEvent("VARIABLES_LOADED");
  elseif ( event == "PLAYER_REGEN_DISABLED" ) then
    NotInCombat = false;
  elseif ( event == "PLAYER_REGEN_ENABLED" ) then
    NotInCombat = true;
  end
end

function CCFocus_MakeMacros()
  _, PlayerClass = UnitClass("player");
  if ( PlayerClass == "MAGE" ) then
    -- Polymorph
    CCFocus_OneMacro( CCFocus_MACRONAME1, CCFocus_Strings.Mage_Spell1, 1, 1 );
    -- Polymorph: Pig
    CCFocus_OneMacro( CCFocus_MACRONAME2, CCFocus_Strings.Mage_Spell2, 1, 1 );
    -- Polymorph: Turtle
    CCFocus_OneMacro( CCFocus_MACRONAME3, CCFocus_Strings.Mage_Spell3, 1, 1 );

  elseif ( PlayerClass ==  "WARLOCK" ) then
    -- Fear
    CCFocus_OneMacro( CCFocus_MACRONAME1, CCFocus_Strings.Warlock_Spell1, 1, 0 );
    -- Banish
    CCFocus_OneMacro( CCFocus_MACRONAME2, CCFocus_Strings.Warlock_Spell2, 1, 0 );
    -- Seduction OR Spell Lock
    CCFocus_OneMacro( CCFocus_MACRONAME3, CCFocus_Strings.Warlock_Spell3, 0, 1, CCFocus_Strings.Seduction );

  elseif ( PlayerClass == "PALADIN" ) then
    -- Turn Evil
    CCFocus_OneMacro( CCFocus_MACRONAME1, CCFocus_Strings.Paladin_Spell1, 1, 0);

  elseif ( PlayerClass == "PRIEST" ) then
    -- Shackle Undead
    CCFocus_OneMacro( CCFocus_MACRONAME1, CCFocus_Strings.Priest_Spell1, 1, 1 );
    -- Mind Control
    CCFocus_OneMacro( CCFocus_MACRONAME2, CCFocus_Strings.Priest_Spell2, 1, 0 );
    -- Silence?

  elseif ( PlayerClass == "DRUID" ) then
    -- Hibernate
    CCFocus_OneMacro( CCFocus_MACRONAME1, CCFocus_Strings.Druid_Spell1, 1, 1 );
    -- Cyclone
    CCFocus_OneMacro( CCFocus_MACRONAME2, CCFocus_Strings.Druid_Spell2, 1, 0 );

  elseif ( PlayerClass == "ROGUE" ) then
    -- Blind
    CCFocus_OneMacro( CCFocus_MACRONAME1, CCFocus_Strings.Rogue_Spell1, 1, 0 );
  
  elseif ( PlayerClass == "HUNTER" ) then
    -- Distracting Shot
    CCFocus_OneMacro( CCFocus_MACRONAME1, CCFocus_Strings.Hunter_Spell1, 1, 1 );
    -- Scatter Shot
    CCFocus_OneMacro( CCFocus_MACRONAME2, CCFocus_Strings.Hunter_Spell2, 1, 0 );
    -- Silencing Shot
    CCFocus_OneMacro( CCFocus_MACRONAME3, CCFocus_Strings.Hunter_Spell3, 1, 0 );
    -- Wyvern Sting
    CCFocus_OneMacro( CCFocus_MACRONAME4, CCFocus_Strings.Hunter_Spell4, 1, 1 );

  end -- of all classes

  CCFocus_MakeRaidMacro(CCFocus_RaidIcon);
end

function CCFocus_MakeRaidMacro(which)
  CCFocus_RaidIcon = which;

  local MacroString = '/script SetRaidTarget("target", '..CCFocus_RaidIcon..');\
/focus';

  local MacroNumber = GetMacroIndexByName(CCFocus_RAIDMACRONAME);
  
  if ( MacroNumber == 0 ) then
    CreateMacro(CCFocus_RAIDMACRONAME, 312, MacroString, 1, 1);
  else
    EditMacro(MacroNumber, CCFocus_RAIDMACRONAME, 312, MacroString, 1, 1);
  end
end

function CCFocus_OneMacro(MacroName, MacroSpell, UseDefaultCondition, Warn, WarnSpell)
  local focusString = "#showtooltip\
/focus [mod:"..CCFocus_Options.SetKey.."]\
/clearfocus [mod:"..CCFocus_Options.ClearKey.."]; [target=focus,dead]\
/stopmacro [mod:"..CCFocus_Options.ClearKey.."]; [nocombat,mod:"..CCFocus_Options.SetKey.."]\
/focus [target=focus,noexists]";

  if ( ( CCWarnLoaded ~= nil ) and ( Warn == 1 ) and ( CCFocus_Options.CCWarn ~= nil ) ) then
    focusString = focusString.."\n/ccfw";
    if ( WarnSpell ~= nil ) then
      focusString = focusString.." "..WarnSpell;
    else
      focusString = focusString.." "..MacroSpell;
    end
  end

  focusString = focusString.."\n/cast ";

  if ( UseDefaultCondition == 1) then
    focusString = focusString.."[target=focus] ";
  end

  focusString = focusString..MacroSpell;

  -- CCFocus_Print("updating or create Macro: "..MacroName)

  local MacroNumber = GetMacroIndexByName(MacroName);

  if ( MacroNumber == 0 ) then
    CreateMacro(MacroName, 1, focusString, 1, 1);
  else 
    EditMacro(MacroNumber, MacroName, 1, focusString, 1, 1);
  end
end

function CCFocus_SetKey(which, key, printme)
  if ( (key ~= "alt") and (key ~= "shift") and (key ~= "ctrl") ) then
    CCFocus_Print(CCFocus_Strings.Mod_Keys_Error);
  else
    if (which == "set") then
      CCFocus_Options.SetKey = key;
      if ( printme == 1 ) then CCFocus_Print(CCFocus_Strings.Mod_Keys_Set..key) end;
    else -- only other option is "clear"
      CCFocus_Options.ClearKey = key;
      if ( printme == 1 ) then CCFocus_Print(CCFocus_Strings.Mod_Keys_Clear..key) end;
    end
    CCFocus_MakeMacros();
  end
end

-- based off code from CCWarn -- calls CCWarn to do work
function CCFocus_Warn(spell)
  if ( spell == "" or spell == nil ) then
    CCWarn("focus", "");
  else
    local usable, nomana = IsUsableSpell(spell);
    local range = not SpellHasRange(spell) or IsSpellInRange(spell, "focus") == 1;
    local cooldown = GetSpellCooldown(spell);
    if usable and not nomana and range and ( not cooldown or cooldown == 0 ) then
      CCWarn("focus", "");
    end
  end
end

function CCFocus_SlashHandler(msg)
  if (msg) then 
    msg = string.lower(msg);
  end
  local args = {};
  for word in string.gmatch(msg, "[^%s]+") do
    table.insert(args, word);
  end

  if ( args[1] == CCFocus_Strings.Cmd_Set and args[2] and NotInCombat ) then
    CCFocus_SetKey("set", args[2], 1);
  elseif ( args[1] == CCFocus_Strings.Cmd_Clear and args[2] and NotInCombat ) then
    CCFocus_SetKey("clear", args[2], 1);
  elseif ( args[1] == CCFocus_Strings.Cmd_Options and NotInCombat ) then
    InterfaceOptionsFrame_OpenToCategory(CCFocusForm)
  elseif ( args[1] == CCFocus_Strings.Cmd_Help ) then
    CCFocus_Print(CCFocus_Strings.Help);
  elseif ( args[1] == CCFocus_Strings.Cmd_Keys ) then
      CCFocus_Print(CCFocus_Strings.Keys1..CCFocus_Options.ClearKey);
      CCFocus_Print(CCFocus_Strings.Keys2..CCFocus_Options.SetKey);
  elseif ( args[1] == "ccwarn" ) then
    if ( args[2] == "on" ) then
      CCFocus_Options.CCWarn = true;
    else
      CCFocus_Options.CCWarn = false;
    end
    CCFocus_MakeMacros();
  else 
    CCFocus_Print(CCFocus_Strings.Usage_Header);
    CCFocus_PrintUsage("Set");
    CCFocus_PrintUsage("Clear");
    CCFocus_PrintUsage("Help");
    CCFocus_PrintUsage("Keys");
    CCFocus_PrintUsage("Options");
    CCFocus_PrintUsage("Usage");
  end 
end
